
import knižnica.*;
import java.awt.Shape;

public class LovecPonoriek extends GRobot
{
	private final static String vyhoďVľavo = "vyhoďVľavo";
	private final static String vyhoďVpravo = "vyhoďVpravo";

	private final Zoznam<Ponorka> ponorky = new Zoznam<>();
	private final Zoznam<Sud> sudy = new Zoznam<>();
	private final Zoznam<Torpédo> torpéda = new Zoznam<>();
	private final Zoznam<Výbuch> výbuchy = new Zoznam<>();

	private int energia = 65;
	private int intervalOpravy = 0;
	private int intervalTorpéd = 0;
	private boolean výhra = false;

	private final static Oblasť tvarLovca;

	static
	{
		Oblasť oblasť = null;

		try
		{
			svgPodpora.vymaž();
			svgPodpora.čítaj("lovec-ponoriek.svg");
			Shape tvar = svgPodpora.dajVýsledný(0);
			oblasť = new Oblasť(SVGPodpora.presuňDoStredu(tvar));
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		tvarLovca = oblasť;
	}

	private void kresliLovca()
	{
		if (výhra)
		{
			hrúbkaČiary(2.5);
			farba(papierová);
			obkresliOblasť(tvarLovca);

			farba(antracitová);
			vyplňOblasť(tvarLovca);
		}
		else
		{
			vyplňOblasť(tvarLovca);

			if (energia < 16) farba(svetločervená);
			else if (energia < 32) farba(tmavožltá);
			else farba(svetlozelená);

			skoč(energia - 65, 17);
			vyplňObdĺžnik(energia, 2);
			skoč(65 - energia, 0);

			farba(čierna);
			kresliObdĺžnik(65, 2);
		}
	}

	private LovecPonoriek()
	{
		farba(tmavotyrkysová);
		odskoč(30);
		vyplňObdĺžnik(400, 270);
		skoč(30);
		Svet.farbaPozadia(tmavooranžová);

		veľkosť(16.5);
		mierka(1);
		vlastnýTvar(r -> ((LovecPonoriek)r).kresliLovca());

		ohranič(Svet.najväčšieX() - 60, Svet.najväčšieY(), PLOT);
		skoč(250);

		zdvihniPero();
		farba(antracitová);

		zrýchleniePosunu(0.1, false);
		maximálnaRýchlosťPosunu(2.5);

		for (int i = 0; i < 9; ++i)
			ponorky.pridaj(new Ponorka());

		for (int i = 0; i < 12; ++i)
			sudy.pridaj(new Sud());

		for (int i = 0; i < 12; ++i)
			torpéda.pridaj(new Torpédo());

		for (int i = 0; i < 30; ++i)
			výbuchy.pridaj(new Výbuch());

		Svet.pridajKlávesovúSkratku(vyhoďVľavo, Kláves.VĽAVO);
		Svet.pridajKlávesovúSkratku(vyhoďVpravo, Kláves.VPRAVO);

		for (Ponorka ponorka : ponorky) ponorka.aktivuj();
		Svet.pridajPoložkuPonuky("Nová hra", Kláves.VK_N, Kláves.VK_N);
	}

	private void nováHra()
	{
		výhra = false;
		deaktivuj(); resetuj();
		for (Sud sud : sudy) sud.deaktivuj();
		for (Torpédo torpédo : torpéda) torpédo.deaktivuj();
		for (Ponorka ponorka : ponorky) ponorka.deaktivuj();
		for (Ponorka ponorka : ponorky) ponorka.aktivuj();
	}

	private void výhra()
	{
		výhra = true;

		zastavPoSpomaleníPosunu();
		if (rýchlosťPosunu() < 0 && zrýchleniePosunu() < 0)
			zrýchleniePosunu(-zrýchleniePosunu());
		else if (rýchlosťPosunu() > 0 && zrýchleniePosunu() > 0)
			zrýchleniePosunu(-zrýchleniePosunu());
	}

	public void resetuj()
	{
		energia = 65;
		intervalOpravy = 0;
		intervalTorpéd = 0;
		polohaX(0);
		rýchlosťPosunu(0);
		zobraz();
	}

	public boolean poškoď(int úroveň)
	{
		if (úroveň >= energia)
		{
			energia = 0;
			intervalOpravy = 0;
			deaktivuj();
			return true;
		}

		energia -= úroveň;
		intervalOpravy = 200;
		return false;
	}

	private void vyhoďVľavo()
	{
		for (Sud sud : sudy)
			if (sud.neaktívny())
			{
				sud.polohaX(polohaX() - 60);
				sud.aktivujVľavo();
				break;
			}
	}

	private void vyhoďVpravo()
	{
		for (Sud sud : sudy)
			if (sud.neaktívny())
			{
				sud.polohaX(polohaX() + 60);
				sud.aktivujVpravo();
				break;
			}
	}

	private void výbuch(Poloha poloha)
	{
		for (Výbuch výbuch : výbuchy)
			if (výbuch.neaktívny())
			{
				výbuch.skočNa(poloha);
				výbuch.aktivuj(10);
				break;
			}
	}

	@Override public void voľbaPoložkyPonuky()
	{
		if (ÁNO == Svet.otázka("Želáte si začať novú hru?")) nováHra();
	}

	@Override public void klávesováSkratka()
	{
		String skratka = ÚdajeUdalostí.príkazSkratky();

		if (!výhra)
		{
			if (skratka == vyhoďVľavo) vyhoďVľavo();
			else if (skratka == vyhoďVpravo) vyhoďVpravo();
		}
	}

	@Override public void stlačenieKlávesu()
	{
		if (!výhra && energia > 0) switch (ÚdajeUdalostí.kláves())
		{
		case Kláves.VPRAVO:
			nezastavujPoSpomaleníPosunu();

			if (zrýchleniePosunu() < 0)
				zrýchleniePosunu(-zrýchleniePosunu());
			else
				aktivuj();

			break;

		case Kláves.VĽAVO:
			nezastavujPoSpomaleníPosunu();

			if (zrýchleniePosunu() > 0)
				zrýchleniePosunu(-zrýchleniePosunu());
			else
				aktivuj();

			break;
		}
	}

	@Override public void uvoľnenieKlávesu()
	{
		if (!výhra && energia > 0) switch (ÚdajeUdalostí.kláves())
		{
		case Kláves.VPRAVO:
		case Kláves.VĽAVO:
			zastavPoSpomaleníPosunu();

			if (rýchlosťPosunu() < 0 && zrýchleniePosunu() < 0)
				zrýchleniePosunu(-zrýchleniePosunu());
			else if (rýchlosťPosunu() > 0 && zrýchleniePosunu() > 0)
				zrýchleniePosunu(-zrýchleniePosunu());

			break;

		case Kláves.MEDZERA:
			vyhoďVľavo(); vyhoďVpravo();
			break;
		}
	}

	@Override public void tik()
	{
		if (výhra) return; boolean výhra = true;
		for (Ponorka ponorka : ponorky) if (ponorka.aktívny())
		{ výhra = false; break; } if (výhra) { výhra(); return; }

		for (Sud sud : sudy) if (sud.aktívny())
		{
			for (Ponorka ponorka : ponorky) if (ponorka.aktívny() &&
				ponorka.bodVElipse(sud, 6.5))
			{
				sud.deaktivuj();
				výbuch(sud);

				if (ponorka.poškoď(13))
				{
					výbuch(ponorka);
					ponorka.preskočVľavo(33);
					výbuch(ponorka);
					ponorka.preskočVpravo(66);
					výbuch(ponorka);
				}
				break;
			}
		}

		if (energia > 0)
		{
			if (energia < 65)
			{
				if (intervalOpravy > 0) --intervalOpravy; else
				{
					intervalOpravy = 30;
					++energia;
				}
			}

			for (Torpédo torpédo : torpéda)
				if (torpédo.aktívny() && bodVElipse(torpédo, 3.7))
				{
					torpédo.deaktivuj();
					výbuch(torpédo);

					if (poškoď(13))
					{
						výbuch(this);
						preskočVľavo(33);
						výbuch(this);
						preskočVpravo(66);
						výbuch(this);
					}
				}

			if (intervalTorpéd > 0) --intervalTorpéd; else
			{
				for (Ponorka ponorka : ponorky) if (ponorka.aktívny() &&
					Math.abs(polohaX() - ponorka.polohaX()) < 60)
					for (Torpédo torpédo : torpéda)
						if (torpédo.neaktívny())
						{
							torpédo.skočNa(ponorka);
							torpédo.aktivuj();
							intervalTorpéd = 300;
							break;
						}
			}
		}
	}

	@Override public void deaktivácia()
	{
		skry();
	}

	public static void main(String[] args)
	{
		Svet.skry();
		Svet.použiKonfiguráciu("LovecPonoriek.cfg");
		Svet.nekresli();
		new LovecPonoriek();
		Svet.kresli();
		Svet.zobraz();
	}
}
