
import knižnica.*;
import java.awt.Shape;

public class Vyšplechnutie extends GRobot
{
	private static class Kvapka extends Častica
	{
		public double vx = 0.0, vy = 3.0, ay = -0.5;

		public void reset(Poloha p, double vx)
		{
			poloha(p);
			uhol = 90;
			this.vx = vx;
			vy = 3.0;
		}

		@Override public void aktivita()
		{
			uhol = Svet.uhol(x, y, x + vx, y + vy);
			x += vx;
			y += vy;
			vy += ay;
		}
	}

	private final Kvapka ľavéKvapky[] = {new Kvapka(),
		new Kvapka(), new Kvapka()};
	private final Kvapka pravéKvapky[] = {new Kvapka(),
		new Kvapka(), new Kvapka()};

	private final static Oblasť kvapkaVľavo;
	private final static Oblasť kvapkaVpravo;

	static
	{
		Oblasť oblasť = null;

		try
		{
			svgPodpora.vymaž();
			svgPodpora.čítaj("kvapka.svg");
			Shape tvar = svgPodpora.dajVýsledný(0);
			oblasť = new Oblasť(SVGPodpora.presuňDoStredu(tvar));
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		kvapkaVľavo = oblasť;
		oblasť = null;

		{
			svgPodpora.pridajTransformácie(0, "scaleX(-1)");
			Shape tvar = svgPodpora.dajVýsledný(0);
			oblasť = new Oblasť(SVGPodpora.presuňDoStredu(tvar));
		}

		kvapkaVpravo = oblasť;
	}

	final static KreslenieTvaru kreslenieVyšplechnutia =
		r -> ((Vyšplechnutie)r).kresliVyšplechnutie();

	private boolean nadHladinou = false;

	private void kresliVyšplechnutie()
	{
		for (Kvapka kvapka : ľavéKvapky)
		{
			skočNa(kvapka);
			smer(kvapka);
			vyplňOblasť(kvapkaVľavo);
		}

		for (Kvapka kvapka : pravéKvapky)
		{
			skočNa(kvapka);
			smer(kvapka);
			vyplňOblasť(kvapkaVpravo);
		}
	}

	public Vyšplechnutie()
	{
		skry();
		vlastnýTvar(kreslenieVyšplechnutia);
		zdvihniPero();
		farba(tmavotyrkysová);
	}

	@Override public void aktivita()
	{
		for (Kvapka kvapka : ľavéKvapky) kvapka.aktivita();
		for (Kvapka kvapka : pravéKvapky) kvapka.aktivita();
	}

	@Override public void aktivácia()
	{
		double
		x = -1.75; for (Kvapka kvapka : ľavéKvapky)
			kvapka.reset(this, x += 0.5);
		x = 1.75; for (Kvapka kvapka : pravéKvapky)
			kvapka.reset(this, x -= 0.5);
		zobraz();
	}

	@Override public void deaktivácia()
	{
		skry();
	}
}
