
import knižnica.*;

public class Stena extends GRobot
{
	private final Plošina plošina;
	private final Lopta lopta;
	private final Zoznam<Tehla> tehly = new Zoznam<>();
	private final Zoznam<Nôž> nožeDole = new Zoznam<>();
	private final Zoznam<Nôž> nožeHore = new Zoznam<>();
	private boolean koniecHry = false;
	private static int nožovNaStreľbu = 60;

	private Stena()
	{
		Svet.farbaPozadia(papierová);
		if (Svet.prvéSpustenie()) Svet.zbaľ();
		skočNa(Svet.najmenšieX() + 3 * veľkosť(),
			Svet.najmenšieY() + veľkosť());

		plošina = new Plošina();
		lopta = new Lopta();
		for (int i = 0; i < 200; ++i) tehly.pridaj(new Tehla());
		Svet.pridajPoložkuPonuky("Nová hra", Kláves.VK_N, Kláves.VK_N);

		lopta.aktivuj();
	}

	private void nováHra()
	{
		for (Tehla tehla : tehly) tehla.reset();
		for (Nôž nôž : nožeDole) nôž.deaktivuj();
		for (Nôž nôž : nožeHore) nôž.deaktivuj();
		plošina.reset(); lopta.reset();
		koniecHry = false;
		nožovNaStreľbu = 60;
	}

	private void prehra()
	{
		koniecHry = true;
		plošina.zastav();
		plošina.nevypĺňajTvary();
	}

	private boolean nováLopta()
	{
		if (lopta.neaktívny())
		{
			if (nožovNaStreľbu >= 0)
			{
				nožovNaStreľbu -= 15;
				lopta.reset();
			}
			else prehra();
			return false;
		}
		return true;
	}

	private Nôž dajNôžDole()
	{
		for (Nôž nôž : nožeDole)
			if (!nôž.aktívny()) return nôž;
		Nôž nôž = new Nôž()
		{
			@Override public void deaktivácia()
			{
				++nožovNaStreľbu;
				skry();
			}
		};
		nožeDole.pridaj(nôž);
		return nôž;
	}

	private Nôž dajNôžHore()
	{
		for (Nôž nôž : nožeHore)
			if (!nôž.aktívny()) return nôž;
		Nôž nôž = new Nôž();
		nôž.farba(červená);
		nôž.smer(SEVER);
		nožeHore.pridaj(nôž);
		return nôž;
	}

	private void novýNôžDole(Poloha p)
	{
		Nôž nôž = dajNôžDole();
		nôž.skočNa(p);
		nôž.aktivuj();
	}

	private boolean novýNôžHore()
	{
		if (nožovNaStreľbu > 0)
		{
			Nôž nôž = dajNôžHore();
			nôž.skočNa(plošina);
			if (0 == nožovNaStreľbu % 2)
				nôž.preskočVpravo(plošina.veľkosť());
			else
				nôž.preskočVľavo(plošina.veľkosť());
			nôž.aktivuj();
			--nožovNaStreľbu;
			return false;
		}
		return true;
	}

	@Override public void kresliTvar()
	{
		if (koniecHry)
		{
			if (nožovNaStreľbu >= 0)
				text("🖒");
			else
				text("🖓");
		}
		else
			text(Svet.celéNaRímske(nožovNaStreľbu));
	}

	@Override public void voľbaPoložkyPonuky()
	{
		if (ÁNO == Svet.otázka("Želáte si začať novú hru?")) nováHra();
	}

	@Override public void tik()
	{
		if (koniecHry) return;

		if (lopta.aktívny())
		{
			if (lopta.koliduje(plošina))
			{
				double Δx = (lopta.polohaX() - plošina.polohaX() +
					Svet.náhodnéReálneČíslo(-1, 1)) / 2;
				lopta.smer(360 - lopta.smer());
				lopta.skoč(lopta.rýchlosť());
				lopta.vpravo(Δx);
			}
		}
		else if (nožovNaStreľbu < 0) prehra();

		boolean jeKoniec = true;
		for (Tehla tehla : tehly)
		{
			if (tehla.viditeľný())
			{
				jeKoniec = false;
				if (lopta.koliduje(tehla))
				{
					double Δx = Math.abs(lopta.polohaX() - tehla.polohaX());
					tehla.skry();
					novýNôžDole(tehla);

					if (Δx < 22)
					{
						lopta.smer(360 - lopta.smer());
						lopta.skoč(lopta.rýchlosť());
					}
					else
					{
						lopta.smer(180 - lopta.smer());
						lopta.skoč(lopta.rýchlosť());
					}

					lopta.vľavo(Svet.náhodnéReálneČíslo(-1, 1));
					break;
				}
				else
				{
					for (Nôž nôž : nožeHore)
					{
						if (nôž.viditeľný() && tehla.koliduje(nôž))
						{
							tehla.skry();
							novýNôžDole(nôž);
							nôž.deaktivuj();
						}
					}
				}
			}
		}

		for (Nôž nôž : nožeDole)
		{
			if (nôž.viditeľný() && plošina.koliduje(nôž))
			{
				plošina.reset();
				lopta.deaktivuj();

				for (Nôž nôž2 : nožeDole)
					if (nôž2.aktívny())
					{
						nožovNaStreľbu -= 3;
						nôž2.deaktivuj();
					}
			}
		}

		if (jeKoniec)
		{
			lopta.deaktivuj();
			plošina.zastav();
			plošina.nevypĺňajTvary();
			koniecHry = true;
		}
	}

	@Override public void stlačenieKlávesu()
	{
		if (koniecHry) return;

		switch (ÚdajeUdalostí.kláves())
		{
		case Kláves.VPRAVO:
			plošina.rozbehniSa();
			break;

		case Kláves.VĽAVO:
			plošina.začniCúvať();
			break;

		case Kláves.MEDZERA:
			if (nováLopta())
				novýNôžHore();
			break;
		}
	}

	@Override public void uvoľnenieKlávesu()
	{
		if (koniecHry) return;

		switch (ÚdajeUdalostí.kláves())
		{
		case Kláves.VPRAVO:
		case Kláves.VĽAVO:
			plošina.zastav();
			break;

		case Kláves.VK_H:
			if (Svet.časovačAktívny())
				Svet.zastavČasovač();
			else
				Svet.spustiČasovač();
		}
	}

	public static void main(String[] args)
	{
		Svet.nekresli();
		try
		{
			Svet.použiKonfiguráciu("Stena.cfg");
			new Stena();
		}
		catch (Throwable t)
		{
			t.printStackTrace();
		}
		Svet.kresli();
	}
}
