
import knižnica.*;

public class MetačBleskov extends GRobot
{
	private final static String[] menáMetač = {"metac-bleskov.svg"};
	private final static SVGPodpora[] svgMetač =
		new SVGPodpora[menáMetač.length];
	private final static int metačŠírka, metačVýška;

	private static double[] poslednýRozmer;
	private static Bod poslednýStred;
	private static double poslednýPosunX = 0.0, poslednýPosunY = 0.0;

	static
	{
		double šírka = -1, výška = -1;
		for (int i = svgMetač.length - 1; i >= 0; --i)
		{
			svgMetač[i] = new SVGPodpora();
			if (prečítajSVG(menáMetač[i], svgMetač[i]))
			{
				if (-1 == šírka || poslednýRozmer[0] > šírka)
				{
					šírka = poslednýRozmer[0];
					poslednýPosunX = poslednýStred.polohaX();
				}
				if (-1 == výška || poslednýRozmer[1] > výška)
				{
					výška = poslednýRozmer[1];
					poslednýPosunY = poslednýStred.polohaY();
				}
			}
		}

		poslednýPosunX += 1;
		poslednýPosunY += -2;

		metačŠírka = (int)(1 + šírka);
		metačVýška = (int)(1 + výška);
	}

	private static boolean prečítajSVG(String súbor, SVGPodpora svgInštancia)
	{
		int prečítané = svgInštancia.čítaj(súbor);

		if (-1 != prečítané)
		{
			poslednýRozmer = svgInštancia.rozmerKresby();
			poslednýStred = svgInštancia.stredKresby();
			return true;
		}

		return false;
	}

	private static void vykresliSVG(GRobot robot, SVGPodpora svgInštancia)
	{
		Farba f = robot.farba();
		double hč = robot.hrúbkaČiary();

		Farba farba;
		int počet = svgInštancia.počet();
		robot.skoč(-poslednýPosunX, -poslednýPosunY);

		for (int i = 0; i < počet; ++i)
		{
			farba = svgInštancia.farbaVýplne(i);
			if (null != farba)
			{
				robot.farba(farba);
				robot.vyplňTvar(svgInštancia.dajVýsledný(i), true);
			}

			farba = svgInštancia.farbaČiary(i);
			if (null != farba)
			{
				double hrúbkaČiary = svgInštancia.hrúbkaČiary(i);
				if (Double.isFinite(hrúbkaČiary))
					robot.hrúbkaČiary(hrúbkaČiary);
				else
					robot.hrúbkaČiary(1);
				robot.farba(farba);
				robot.kresliTvar(svgInštancia.dajVýsledný(i), true);
			}
		}

		robot.skoč(poslednýPosunX, poslednýPosunY);
		robot.hrúbkaČiary(hč);
		robot.farba(f);
	}


	private Obrázok tvar;
	private int elektrizujem = 0;
	private int nabíjam = 0;
	private int posun = 0;


	public MetačBleskov()
	{
		skry();
		zdvihniPero();
		veľkosť(metačŠírka * 2);
		mierka(1);

		// Vytvorenie obrázka na nakreslenie základného tvaru:
		tvar = new Obrázok(metačŠírka, metačVýška);
		kresliDoObrázka(tvar);
		vykresliSVG(this, svgMetač[0]);
		kresliNaPodlahu();
		hrúbkaČiary(1.5);
		farba(svetlotyrkysová);

		zobraz();
		zrýchlenie(0.1, false);
		maximálnaRýchlosť(5);
		uhlovéZrýchlenie(0.1, false);
		dosiahnutieCieľa();
	}


	public void nastavPosun(int posun)
	{
		nabíjam = 240;
		this.posun = posun;
		upravCieľ(cieľX() + posun, cieľY());
		polohaX(polohaX() + posun);
	}

	public void posuňMetač(int posun)
	{
		this.posun += posun;
		polohaX(polohaX() + posun);
		upravCieľ(cieľX() + posun, cieľY());
	}

	public boolean elektrizuje()
	{
		return elektrizujem > 0;
	}

	public void elektrizuj()
	{
		if (nabíjam <= 0)
		{
			nabíjam = 120;
			elektrizujem = 60;
		}
	}


	@Override public void kresliTvar()
	{
		obrázok(tvar);

		if (elektrizujem > 0)
		{
			double dĺžka = veľkosť() * 0.25;
			double vysunutie = dĺžka * 0.50;
			Bod[] body = new Bod[4];
			Bod počiatok = poloha();
			for (int i = 0; i < 8; ++i)
			{
				skočNa(počiatok);
				for (int j = 0; j < 4; ++j)
				{
					skoč(dĺžka);
					double vysuň = Svet.
						náhodnéReálneČíslo(-vysunutie, vysunutie);
					preskočVpravo(vysuň);
					body[j] = poloha();
					preskočVľavo(vysuň);
				}

				skočNa(počiatok);
				for (int j = 0; j < 4; ++j)
					choďNa(body[j]);

				vpravo();
			}
		}
	}

	@Override public void tik()
	{
		if (nabíjam > 0) --nabíjam;
		if (elektrizujem > 0) --elektrizujem;
	}

	@Override public void dosiahnutieCieľa()
	{
		Bod poloha = poloha();
		náhodnáPoloha();
		Bod cieľ = poloha();
		cieľ.polohaX(cieľ.polohaX() + posun);
		poloha(poloha);
		cieľ(cieľ);
	}
}
