
import java.awt.geom.AffineTransform;
import knižnica.*;

public class Zberateľ extends GRobot
{
	private final static String[] menáZberateľ = {"zberatel-stoji-vpravo.svg",
		"zberatel-ide-1-vpravo.svg", "zberatel-ide-2-vpravo.svg"};
	private final static SVGPodpora[] svgZberateľ =
		new SVGPodpora[menáZberateľ.length];
	private final static int zberateľŠírka, zberateľVýška;

	private static double[] poslednýRozmer;
	private static Bod poslednýStred;
	private static double poslednýPosunX = 0.0, poslednýPosunY = 0.0;

	static
	{
		double šírka = -1, výška = -1;
		for (int i = svgZberateľ.length - 1; i >= 0; --i)
		{
			svgZberateľ[i] = new SVGPodpora();
			if (prečítajSVG(menáZberateľ[i], svgZberateľ[i]))
			{
				if (-1 == šírka || poslednýRozmer[0] > šírka)
				{
					šírka = poslednýRozmer[0];
					poslednýPosunX = poslednýStred.polohaX();
				}
				if (-1 == výška || poslednýRozmer[1] > výška)
				{
					výška = poslednýRozmer[1];
					poslednýPosunY = poslednýStred.polohaY();
				}
			}
		}

		poslednýPosunX -= 3;
		zberateľŠírka = (int)(1 + šírka);
		zberateľVýška = (int)(1 + výška);
	}

	private static boolean prečítajSVG(String súbor, SVGPodpora svgInštancia)
	{
		int prečítané = svgInštancia.čítaj(súbor);

		if (-1 != prečítané)
		{
			poslednýRozmer = svgInštancia.rozmerKresby();
			poslednýStred = svgInštancia.stredKresby();
			return true;
		}

		return false;
	}

	private static void vykresliSVG(GRobot robot, SVGPodpora svgInštancia)
	{
		Farba f = robot.farba();
		double hč = robot.hrúbkaČiary();

		Farba farba;
		int počet = svgInštancia.počet();
		robot.skoč(-poslednýPosunX, -poslednýPosunY);

		for (int i = 0; i < počet; ++i)
		{
			farba = svgInštancia.farbaVýplne(i);
			if (null != farba)
			{
				robot.farba(farba);
				robot.vyplňTvar(svgInštancia.dajVýsledný(i), true);
			}

			farba = svgInštancia.farbaČiary(i);
			if (null != farba)
			{
				double hrúbkaČiary = svgInštancia.hrúbkaČiary(i);
				if (Double.isFinite(hrúbkaČiary))
					robot.hrúbkaČiary(hrúbkaČiary);
				else
					robot.hrúbkaČiary(1);
				robot.farba(farba);
				robot.kresliTvar(svgInštancia.dajVýsledný(i), true);
			}
		}

		robot.skoč(poslednýPosunX, poslednýPosunY);
		robot.hrúbkaČiary(hč);
		robot.farba(f);
	}


	private Obrázok kráčajVpravo, kráčajVľavo, aktívnyTvar;
	private int poklesNôh, vysunutieNôh;
	private boolean môžeSkočiť = true;
	private int energia = 50;
	private int liečSa = 100;

	public Zberateľ()
	{
		zdvihniPero();
		veľkosť(zberateľŠírka);
		mierka(1);

		poklesNôh = zberateľVýška / 2;
		vysunutieNôh = zberateľŠírka / 4;

		// Vytvorenie obrázkov vlastných tvarov:
		kráčajVpravo = new Obrázok(zberateľŠírka, zberateľVýška);
		kráčajVľavo = new Obrázok(zberateľŠírka, zberateľVýška);
		kráčajVpravo.trvanie(0.060);
		kráčajVľavo.trvanie(0.060);
		kresliDoObrázka(kráčajVpravo);

		// Vykreslenie prvej snímky kráčania:
		kráčajVpravo.vymaž();
		vykresliSVG(this, svgZberateľ[0]);
		kráčajVpravo.pridajSnímku();

		kráčajVľavo.vymaž();
		kresliDoObrázka(kráčajVľavo);
		obrázok(kráčajVpravo);
		kresliDoObrázka(kráčajVpravo);
		kráčajVľavo.prevráťZvislo();
		kráčajVľavo.pridajSnímku();

		// Vykreslenie druhej snímky kráčania:
		kráčajVpravo.vymaž();
		vykresliSVG(this, svgZberateľ[1]);
		kráčajVpravo.pridajSnímku();

		kráčajVľavo.vymaž();
		kresliDoObrázka(kráčajVľavo);
		obrázok(kráčajVpravo);
		kresliDoObrázka(kráčajVpravo);
		kráčajVľavo.prevráťZvislo();
		kráčajVľavo.pridajSnímku();

		// Vykreslenie tretej snímky kráčania:
		kráčajVpravo.vymaž();
		vykresliSVG(this, svgZberateľ[0]);
		kráčajVpravo.pridajSnímku();

		kráčajVľavo.vymaž();
		kresliDoObrázka(kráčajVľavo);
		obrázok(kráčajVpravo);
		kresliDoObrázka(kráčajVpravo);
		kráčajVľavo.prevráťZvislo();
		kráčajVľavo.pridajSnímku();

		// Vykreslenie štvrtej snímky kráčania:
		kráčajVpravo.vymaž();
		vykresliSVG(this, svgZberateľ[2]);
		kráčajVpravo.pridajSnímku();

		kráčajVľavo.vymaž();
		kresliDoObrázka(kráčajVľavo);
		obrázok(kráčajVpravo);
		kresliDoObrázka(kráčajVpravo);
		kráčajVľavo.prevráťZvislo();
		kráčajVľavo.pridajSnímku();

		// Nastavenie ďalších vlastností:
		kresliNaPodlahu();
		rýchlosťPosunu(0, false);
		zrýchlenie(-1, false);
		maximálnaRýchlosť(20);
		otočDoprava();

		skoč(poklesNôh * 2);
		ohranič(Svet.šírka() / 2 - 100, Svet.výška() / 2 + poklesNôh, PLOT);
		odskoč(poklesNôh * 2);
	}


	public void skočNa(Platforma platforma)
	{
		polohaY(platforma.úroveňZeme() + poklesNôh);
	}

	public Bod noha1()
	{
		return new Bod(polohaX() - vysunutieNôh, polohaY() - poklesNôh);
	}

	public Bod noha2()
	{
		return new Bod(polohaX() + vysunutieNôh, polohaY() - poklesNôh);
	}

	public void zastavSa()
	{
		aktívnyTvar.zastav();
		aktívnyTvar.snímka(0);
		rýchlosťPosunu(0);
	}

	public void otočDoprava()
	{
		aktívnyTvar = kráčajVpravo;
		aktívnyTvar.snímka(0);
		vlastnýTvar(aktívnyTvar);
	}

	public void otočDoľava()
	{
		aktívnyTvar = kráčajVľavo;
		aktívnyTvar.snímka(0);
		vlastnýTvar(aktívnyTvar);
	}

	public void kráčajDoprava()
	{
		rýchlosťPosunu(10);
		aktívnyTvar = kráčajVpravo;
		aktívnyTvar.spusti();
		aktívnyTvar.snímka(1);
		vlastnýTvar(aktívnyTvar);
	}

	public void kráčajDoľava()
	{
		rýchlosťPosunu(-10);
		aktívnyTvar = kráčajVľavo;
		aktívnyTvar.spusti();
		aktívnyTvar.snímka(1);
		vlastnýTvar(aktívnyTvar);
	}

	public void dopadni()
	{
		rýchlosť(0, false);
		môžeSkočiť = true;
	}

	public boolean zraň()
	{
		if (energia > 0)
		{
			--energia;
			priehľadnosť(0.10 + (energia / 56.0));
		}
		else if (viditeľný())
		{
			deaktivuj();
			return true;
		}
		return false;
	}

	public void reset()
	{
		energia = 50;
		priehľadnosť(1.0);
		skočNa(0, 0);
		zastavSa();
		dopadni();
		aktivuj();
	}


	@Override public void tik()
	{
		if (energia > 0 && energia < 50)
		{
			if (liečSa > 0) --liečSa; else
			{
				liečSa = 100;
				if (50 == ++energia)
					priehľadnosť(1.0);
				else
					priehľadnosť(0.10 + (energia / 56.0));
			}
		}
	}

	@Override public void aktivácia()
	{
		zobraz();
	}

	@Override public void deaktivácia()
	{
		skry();
	}


	@Override public boolean mimoHraníc(Bod[] poleBodov, double uhol)
	{
		if (poleBodov[2].polohaY() == poleBodov[3].polohaY())
		{
			if (poleBodov[2].polohaY() > 0) return false; else dopadni();
		}
		else if (poleBodov[2].polohaX() == poleBodov[3].polohaX())
		{
			if (poleBodov[2].polohaX() < 0)
				ZberateľMincí.posuňProstredie(10);
			else
				ZberateľMincí.posuňProstredie(-10);
			return false;
		}
		return true;
	}

	@Override public void stlačenieKlávesu()
	{
		if (neaktívny()) return;
		switch (ÚdajeUdalostí.kláves())
		{
		case Kláves.VPRAVO:
			if (0 == rýchlosťPosunu()) kráčajDoprava();
			break;

		case Kláves.VĽAVO:
			if (0 == rýchlosťPosunu()) kráčajDoľava();
			break;

		case Kláves.MEDZERA:
			if (môžeSkočiť && 0 == rýchlosť())
			{
				rýchlosť(20, false);
				môžeSkočiť = false;
			}
		}
	}

	@Override public void uvoľnenieKlávesu()
	{
		if (neaktívny()) return;
		switch (ÚdajeUdalostí.kláves())
		{
		case Kláves.VPRAVO:
		case Kláves.VĽAVO:
			zastavSa();
			break;
		}
	}
}
